{
  "type": "record",
  "Aspect": {
    "name": "oauthAuthorizationServerKey",
    "keyForEntity": "oauthAuthorizationServer",
    "entityCategory": "core",
    "entityAspects": [
      "oauthAuthorizationServerProperties",
      "ownership",
      "status"
    ],
    "entityDoc": "An OAuth Authorization Server that DataHub can use to obtain tokens for calling external APIs."
  },
  "name": "OAuthAuthorizationServerKey",
  "namespace": "com.linkedin.pegasus2avro.oauth",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "Unique identifier for this authorization server.\nExamples: glean, snowflake, github"
    }
  ],
  "doc": "Key for an OAuth Authorization Server entity.\n\nAn OAuth Authorization Server represents an external OAuth provider\nthat DataHub can use to obtain tokens for calling external APIs.\nThis is for OUTBOUND OAuth (DataHub as client), not INBOUND auth\n(users authenticating to DataHub)."
}