{
  "type": "record",
  "Aspect": {
    "name": "assertionsSummary"
  },
  "name": "AssertionsSummary",
  "namespace": "com.linkedin.pegasus2avro.common",
  "fields": [
    {
      "deprecated": true,
      "type": {
        "type": "array",
        "items": "string"
      },
      "name": "failingAssertions",
      "default": [],
      "doc": "Failing assertions for an asset\nDeprecated! Use failingAssertionDetails.",
      "Urn": "Urn",
      "urn_is_array": true
    },
    {
      "deprecated": true,
      "type": {
        "type": "array",
        "items": "string"
      },
      "name": "passingAssertions",
      "default": [],
      "doc": "Passing assertions for an asset\nDeprecated! Use passingAssertionDetails.",
      "Urn": "Urn",
      "urn_is_array": true
    },
    {
      "Searchable": {
        "fieldName": "lastAssertionResultAt",
        "fieldType": "DATETIME",
        "queryByDefault": false
      },
      "type": [
        "null",
        "long"
      ],
      "name": "lastAssertionResultAt",
      "default": null,
      "doc": "The last time an assertion was run."
    },
    {
      "Relationship": {
        "/*/urn": {
          "entityTypes": [
            "assertion"
          ],
          "name": "PassingAssertions"
        }
      },
      "Searchable": {
        "/*/lastResultAt": {
          "fieldName": "passingAssertionResultTimes",
          "fieldType": "DATETIME",
          "queryByDefault": false
        },
        "/*/source": {
          "fieldName": "passingAssertionSources",
          "queryByDefault": false
        },
        "/*/type": {
          "fieldName": "passingAssertionTypes",
          "fieldType": "KEYWORD",
          "queryByDefault": false
        },
        "/*/urn": {
          "fieldName": "passingAssertions",
          "fieldType": "URN",
          "hasValuesFieldName": "hasPassingAssertions",
          "numValuesFieldName": "numPassingAssertions",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": {
          "type": "record",
          "name": "AssertionSummaryDetails",
          "namespace": "com.linkedin.pegasus2avro.common",
          "fields": [
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": "string",
              "name": "urn",
              "doc": "The urn of the assertion",
              "Urn": "Urn"
            },
            {
              "type": "string",
              "name": "type",
              "doc": "The top-level type of an assertion"
            },
            {
              "type": "long",
              "name": "lastResultAt",
              "doc": "The time at which the last result was produced."
            },
            {
              "type": [
                "null",
                "string"
              ],
              "name": "source",
              "default": null,
              "doc": "The source of the assertion"
            }
          ],
          "doc": "Summary statistics about assertions on an entity."
        }
      },
      "name": "passingAssertionDetails",
      "default": [],
      "doc": "Summary details about the set of passing assertions"
    },
    {
      "Relationship": {
        "/*/urn": {
          "entityTypes": [
            "assertion"
          ],
          "name": "FailingAssertions"
        }
      },
      "Searchable": {
        "/*/lastResultAt": {
          "fieldName": "failingAssertionResultTimes",
          "fieldType": "DATETIME",
          "queryByDefault": false
        },
        "/*/source": {
          "fieldName": "failingAssertionSources",
          "queryByDefault": false
        },
        "/*/type": {
          "fieldName": "failingAssertionType",
          "fieldType": "KEYWORD",
          "queryByDefault": false
        },
        "/*/urn": {
          "addHasValuesToFilters": true,
          "fieldName": "failingAssertions",
          "fieldType": "URN",
          "hasValuesFieldName": "hasFailingAssertions",
          "numValuesFieldName": "numFailingAssertions",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": "com.linkedin.pegasus2avro.common.AssertionSummaryDetails"
      },
      "name": "failingAssertionDetails",
      "default": [],
      "doc": "Summary details about the set of failing assertions"
    },
    {
      "Relationship": {
        "/*/urn": {
          "entityTypes": [
            "assertion"
          ],
          "name": "ErroringAssertions"
        }
      },
      "Searchable": {
        "/*/lastResultAt": {
          "fieldName": "erroringAssertionResultTimes",
          "fieldType": "DATETIME"
        },
        "/*/source": {
          "fieldName": "erroringAssertionSources",
          "queryByDefault": false
        },
        "/*/type": {
          "fieldName": "erroringAssertionType",
          "fieldType": "KEYWORD"
        },
        "/*/urn": {
          "fieldName": "erroringAssertions",
          "fieldType": "URN",
          "hasValuesFieldName": "hasErroringAssertions",
          "numValuesFieldName": "numErroringAssertions",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": "com.linkedin.pegasus2avro.common.AssertionSummaryDetails"
      },
      "name": "erroringAssertionDetails",
      "default": [],
      "doc": "Summary details about the set of erroring assertions"
    },
    {
      "Relationship": {
        "/*/urn": {
          "entityTypes": [
            "assertion"
          ],
          "name": "InitializingAssertions"
        }
      },
      "Searchable": {
        "/*/lastResultAt": {
          "fieldName": "initializingAssertionResultTimes",
          "fieldType": "DATETIME",
          "queryByDefault": false
        },
        "/*/source": {
          "fieldName": "initializingAssertionSources",
          "queryByDefault": false
        },
        "/*/type": {
          "fieldName": "initializingAssertionType",
          "fieldType": "KEYWORD",
          "queryByDefault": false
        },
        "/*/urn": {
          "fieldName": "initializingAssertions",
          "fieldType": "URN",
          "hasValuesFieldName": "hasInitializingAssertions",
          "numValuesFieldName": "numInitializingAssertions",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": "com.linkedin.pegasus2avro.common.AssertionSummaryDetails"
      },
      "name": "initializingAssertionDetails",
      "default": [],
      "doc": "Summary details about the set of initializing assertions"
    }
  ],
  "doc": "Summary related to entity assertions"
}