{
  "type": "record",
  "Aspect": {
    "name": "dataHubRemoteExecutorPoolInfo"
  },
  "name": "RemoteExecutorPoolInfo",
  "namespace": "com.linkedin.pegasus2avro.executorpool",
  "fields": [
    {
      "Searchable": {
        "fieldType": "COUNT",
        "queryByDefault": false
      },
      "type": "long",
      "name": "createdAt",
      "doc": "UTC-based timestamp of when this pool was created"
    },
    {
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
      },
      "type": [
        "null",
        "string"
      ],
      "name": "creator",
      "default": null,
      "doc": "The creator of this pool",
      "Urn": "Urn"
    },
    {
      "type": [
        "null",
        "string"
      ],
      "name": "description",
      "default": null,
      "doc": "A description for this pool"
    },
    {
      "type": [
        "null",
        "string"
      ],
      "name": "queueUrl",
      "default": null,
      "doc": "The url to the task queue for this pool. I.e., SQS queue url."
    },
    {
      "type": [
        "null",
        "string"
      ],
      "name": "queueRegion",
      "default": null,
      "doc": "The region of the sqs queue for this pool for backwards compatibility."
    },
    {
      "type": [
        "null",
        "boolean"
      ],
      "name": "isEmbedded",
      "default": null,
      "doc": "Only set true if this is the pool embedded within the DataHub Cloud deployment"
    },
    {
      "type": [
        "null",
        {
          "type": "record",
          "name": "RemoteExecutorPoolState",
          "namespace": "com.linkedin.pegasus2avro.executorpool",
          "fields": [
            {
              "type": {
                "type": "enum",
                "symbolDocs": {
                  "PROVISIONING_FAILED": "The pool has failed to be provisioned.",
                  "PROVISIONING_IN_PROGRESS": "The pool has been picked up by DataHub and is in the process of being provisioned.",
                  "PROVISIONING_PENDING": "The pool is pending provisioning. Default state on creation.",
                  "READY": "The pool has been successfully provisioned and is ready to accept tasks."
                },
                "name": "RemoteExecutorPoolStatus",
                "namespace": "com.linkedin.pegasus2avro.executorpool",
                "symbols": [
                  "PROVISIONING_PENDING",
                  "PROVISIONING_IN_PROGRESS",
                  "PROVISIONING_FAILED",
                  "READY"
                ]
              },
              "name": "status",
              "doc": "The status of the remote executor pool"
            },
            {
              "type": [
                "null",
                "string"
              ],
              "name": "message",
              "default": null,
              "doc": "The message associated with the status. I.e., an error message explaining failure."
            }
          ]
        }
      ],
      "name": "state",
      "default": null,
      "doc": "The status of the remote executor pool"
    }
  ]
}